package webservice.reservationservice.reservationservice;
/**
 * Source code created on Wed Sep 20 16:16:55 EDT 2006
 */

import com.sun.rave.designtime.*;
import com.sun.rave.designtime.impl.*;
import java.util.ArrayList;

public class ReservationServiceReserveDesignInfo extends BasicDesignInfo {

    public ReservationServiceReserveDesignInfo() {
        super( ReservationServiceClient.class );
    } 

    public Result beanCreatedSetup( DesignBean designBean ) {
        String currentScope = (String)designBean.getDesignContext().getContextData( Constants.ContextData.SCOPE );
        ArrayList clientBeans = new ArrayList();
        DesignContext[] contexts = designBean.getDesignContext().getProject().getDesignContexts();
        for( int i = 0; i < contexts.length; i ++ ) {
            DesignBean[] beans = contexts[i].getBeansOfType( ReservationServiceClient.class );
            for( int bi = 0; bi < beans.length; bi ++ ) {

                // Filter out the ones in the same scope unless it is in the same backing bean
                String sourceBeanScope = (String)beans[bi].getDesignContext().getContextData( Constants.ContextData.SCOPE );
                if( currentScope.equals( sourceBeanScope ) && !designBean.getBeanParent().getInstanceName().equals( beans[bi].getBeanParent().getInstanceName() ) )
                    continue;

                // Filter out the ones in smaller scopes.
                // For example, if the current scope is in application, then only the ones in the application scope can be referred 
                if( currentScope.equals( "application" ) && !sourceBeanScope.equals( "application" ) )
                    continue;

                // If the current scope is in session, then only the ones in application and session can be referred.
                // In another words, the ones from request are not good
                if( currentScope.equals( "session" ) && sourceBeanScope.equals( "request" ) )
                    continue;

                clientBeans.add( beans[bi] );
            }
        }
        if( clientBeans.size() == 0 ) { 
            if( designBean.getDesignContext().canCreateBean( "webservice.reservationservice.reservationservice.ReservationServiceClient", designBean.getBeanParent(), null ) ) {
                DesignBean clientBean = designBean.getDesignContext().createBean( "webservice.reservationservice.reservationservice.ReservationServiceClient", designBean.getBeanParent(),  null );
                designBean.getProperty( "reservationServiceClient").setValue( clientBean.getInstance() );
                return Result.SUCCESS;
            }
            else {

                ResultMessage message = new ResultMessage( ResultMessage.TYPE_CRITICAL, "No GreeterClient instance", "No GreeterClient instance found and failed to create one" );
                return new Result( true, message );
             }
         } else if( clientBeans.size() == 1 ) {
             DesignBean selectedBean = (DesignBean)clientBeans.get(0);
             if( designBean.getBeanParent().getInstanceName().equals( selectedBean.getBeanParent().getInstanceName() ) ) {
                 designBean.getProperty( "reservationServiceClient").setValue( ((DesignBean)clientBeans.get(0)).getInstance() );
             }
             else
             {
                 String pInstanceName = capitalize( selectedBean.getBeanParent().getInstanceName() );
                 String selectedInstanceName = capitalize( selectedBean.getInstanceName() );

                  String sourceBeanStr = "get" + pInstanceName + "().get" + selectedInstanceName + "()"; // NOI18N
                  designBean.getProperty( "reservationServiceClient").setValueSource( sourceBeanStr );
             }

             return Result.SUCCESS;
         } else {// More than one found
             DesignProperty prop = designBean.getProperty( "reservationServiceClient" );
             Customizer2 dpCustomizer = new DataClassInstanceCustomizer( "DataProvider customizer", designBean, prop, (DesignBean[])clientBeans.toArray(new DesignBean[0]) ); 
             return new CustomizerResult( designBean, dpCustomizer );
         }
     }

     private String capitalize(String name) {
         if( name == null || name.length() == 0 )
              return name;

         char chars[] = name.toCharArray();
         chars[0] = Character.toUpperCase(chars[0]);
         return new String(chars);
    }

    /**
     * This customizer for setting the data class instance on a data provider
     */
    public class DataClassInstanceCustomizer extends BasicCustomizer2 {
        // The bean where the property from
        protected DesignBean targetBean;

        // The design property to be set
        protected DesignProperty targetProperty;

        // The bean instance from one the following design beans will be set to the target property
        protected DesignBean[] sourceBeans;

        // The List to display the bean instance to be selected from
        private javax.swing.JList beanInstancesList;

        /**
         * Constructor
         *
         * @param name The title of the customizer
         * @param targetProperty The property this customizer is designed to set
         * @param sourceBeans
         */
        public DataClassInstanceCustomizer(String name, DesignBean targetBean, DesignProperty targetProperty, DesignBean[] sourceBeans) { 
            super( null, name ); 
            setApplyCapable( false );
            this.targetProperty = targetProperty;
            this.targetBean = targetBean;
            this.sourceBeans = sourceBeans;
        }

        protected java.awt.Component createCustomizerPanel() {

            javax.swing.JPanel panel = new javax.swing.JPanel();

            java.awt.GridBagConstraints gridBagConstraints;

            javax.swing.JScrollPane listScrollPane = new javax.swing.JScrollPane();
            beanInstancesList = new javax.swing.JList();
            javax.swing.JLabel label = new javax.swing.JLabel();

            panel.setLayout(new java.awt.GridBagLayout());

            beanInstancesList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
            beanInstancesList.setVisibleRowCount(4);
            listScrollPane.setViewportView(beanInstancesList);

            gridBagConstraints = new java.awt.GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = 1;
            gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
            gridBagConstraints.insets = new java.awt.Insets(12, 12, 12, 12);
            gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
            gridBagConstraints.weightx = 1.0;
            gridBagConstraints.weighty = 1.0;
            panel.add(listScrollPane, gridBagConstraints);

            label.setText("Select a ReservationServiceClient instance to set on data provider instance " + targetBean.getInstanceName() );
            gridBagConstraints = new java.awt.GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = 0;
            gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
            gridBagConstraints.insets = new java.awt.Insets(12, 12, 0, 12);
            gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
            panel.add(label, gridBagConstraints);

            String[] beanInstanceNames = new String[ sourceBeans.length ];
            for( int i = 0; i < sourceBeans.length; i ++ ) {
                String parentInstance = sourceBeans[i].getBeanParent().getInstanceName();
                beanInstanceNames[i] = sourceBeans[i].getInstanceName() + "(" + parentInstance + ")";
            }
            beanInstancesList.setListData( beanInstanceNames );

// Have the first one selected by default
            beanInstancesList.setSelectedIndex(0);

            return panel;
        }

        public Result applyChanges() {

            int index = beanInstancesList.getSelectedIndex();
            DesignBean selectedBean = sourceBeans[index];
            if( designBean.getBeanParent().getInstanceName().equals( selectedBean.getBeanParent().getInstanceName() ) ) {
                targetProperty.setValue( sourceBeans[index].getInstance() );
            }
            else {
                String pInstanceName = capitalize( selectedBean.getBeanParent().getInstanceName() );
                String selectedInstanceName = capitalize( selectedBean.getInstanceName() );

                String sourceBeanStr = "get" + pInstanceName + "().get" + selectedInstanceName + "()"; // NOI18N
                targetProperty.setValueSource( sourceBeanStr );
            }

            return Result.SUCCESS;
        }
    }
}
